@@ -34,34 +34,36 @@ import gobject
 import pango
 import pycentrino_globals as pglobals
 
-def delete_node(nodo):
-    if hasattr(nodo,get_children):
-        if nodo.get_children()!= []:
-            nodo.remove(nodo.get_children()[0])
-            delete_node(nodo)
-        else:
-            del nodo
-            return
-    else:
-        del nodo
-        return
- 
-def delete_widget(widget):
-    parent = None
-    if hasattr(widget,'get_children'):
-        for child in widget.get_children():
-            print "borrando relacion " + str(widget) + str(child)
-            widget.remove(child)
-            delete_widget(child)
-            parent = widget.get_parent()
-        if parent != None:
-            parent.remove(widget)
-
 class pycentrinoApplet(gnome.applet.Applet):
         
     def cleanup(self,event):
         del self.applet
 
+    # add to history
+    def add_to_history(self,element):
+        if len(self.history) < self.history_max:
+            self.history.append(list(element))
+        else:
+            self.history.pop()
+            self.history.append(list(element))
+
+    # function to compare only the values shown at the moment (checks[])
+    def comp_history(self,actual):
+        for i in range(len(actual)):
+            if self.checks[i]:
+                if actual[i] != self.history[len(self.history)-1][i]:
+                    return True
+        return False
+                
+                
+    # function to see if any info has changed since last time in self.history queue
+    def info_has_changed(self,actual):
+        
+        if self.comp_history(actual):
+            return True
+        else:
+            return False
+        
     # font definition
     def define_font(self):
         fd = pango.FontDescription()
@@ -73,65 +75,71 @@ class pycentrinoApplet(gnome.applet.Applet):
         
     # this callback is for update the applet from system info every 1 second
     def timeout_callback(self,event):
-
+        self.add_to_history(self.info)
         self.read_proc_info()
-        self.orientation = self.applet.get_orient()
-        self.size = self.applet.get_size()
+        
+        if self.info_has_changed(self.info):
 
-        # when we delete the children, they run out of references,
-        # so the garbage collector must delete them totally
-        for i in (self.box.get_children()):
-            self.box.remove(i)
+            # when we delete the children, they run out of references,
+            # so the garbage collector must delete them totally
+            for i in (self.box.get_children()):
+                self.box.remove(i)
 
-        cont = -1
-        separate = 0
-        # create the applet with little hboxes containing each info checked in the preferences
-        for i in range(len(self.checks)):
-            if self.checks[i]:
-                cont = cont + 1
+            cont = -1
+            separate = 0
+            # create the applet with little hboxes containing each info checked in the preferences
+            for i in range(len(self.checks)):
+                if self.checks[i]:
+                    cont = cont + 1
 
-                self.box.pack_start(gtk.EventBox())
+                    self.box.pack_start(gtk.EventBox())
 
-                # add un hbox para meter contenido
-                self.box.get_children()[cont].add(gtk.HBox())
+                    # add un hbox para meter contenido
+                    self.box.get_children()[cont].add(gtk.HBox())
 
-                # metemos una etiqueta con un espacio
-                tmp = gtk.Label(" ")
-                self.box.get_children()[cont].get_children()[0].pack_start(tmp)
-
-                cont = cont + 1
-                self.box.pack_start(gtk.EventBox())
-
-                # add un hbox para meter contenido
-                self.box.get_children()[cont].add(gtk.HBox())
-                if i==3:
-                    if self.info[i] == "powersave":
-                        tmp = gtk.Image()
-                        tmp.set_from_file(os.path.join(pglobals.image_dir,'powersave.png'))
-                        self.box.get_children()[cont].get_children()[0].pack_start(tmp)
-                    elif self.info[i] == "performance":
-                        tmp = gtk.Image()
-                        tmp.set_from_file(os.path.join(pglobals.image_dir,'performance.png'))
-                        self.box.get_children()[cont].get_children()[0].pack_start(tmp)
-                    elif self.info[i] == "userspace":
-                        tmp = gtk.Image()
-                        tmp.set_from_file(os.path.join(pglobals.image_dir,'userspace.png'))
-                        self.box.get_children()[cont].get_children()[0].pack_start(tmp)
-
-                # si metemos la informacin de que procesador es, metemos imagen correspondiente
-                elif i==4:
-                    if self.info[i] == "centrino":
-                        tmp = gtk.Image()
-                        tmp.set_from_file(os.path.join(pglobals.image_dir,'centrino_46.png'))
-                        self.box.get_children()[cont].get_children()[0].pack_start(tmp)
-                else:
-                    fd = self.define_font()
-                    
-                    tmp = gtk.Label(self.info[i])
-                    tmp.modify_font(fd)
+                    # metemos una etiqueta con un espacio
+                    tmp = gtk.Label(" ")
                     self.box.get_children()[cont].get_children()[0].pack_start(tmp)
 
-        #self.applet.hide()
+                    cont = cont + 1
+                    self.box.pack_start(gtk.EventBox())
+
+                    # add un hbox para meter contenido
+                    self.box.get_children()[cont].add(gtk.HBox())
+                    tooltip = gtk.Tooltips()
+                    if i == 3:
+                        tmp = gtk.Image()
+                        if self.info[i] == "":
+                            tooltip.set_tip(self.box.get_children()[cont], "no tiene soporte para cpufreq")
+                            tmp.set_from_file(os.path.join(pglobals.image_dir,'warning.png'))
+                            self.box.get_children()[cont].get_children()[0].pack_start(tmp)
+                        if self.info[i] == "powersave":
+                            tooltip.set_tip(self.box.get_children()[cont], "ahorro de energia")
+                            tmp.set_from_file(os.path.join(pglobals.image_dir,'powersave.png'))
+                            self.box.get_children()[cont].get_children()[0].pack_start(tmp)
+                        elif self.info[i] == "performance":
+                            tooltip.set_tip(self.box.get_children()[cont], "energia al maximo")
+                            tmp.set_from_file(os.path.join(pglobals.image_dir,'performance.png'))
+                            self.box.get_children()[cont].get_children()[0].pack_start(tmp)
+                        elif self.info[i] == "userspace":
+                            tooltip.set_tip(self.box.get_children()[cont], "Usando modo usuario")
+                            tmp.set_from_file(os.path.join(pglobals.image_dir,'userspace.png'))
+                            self.box.get_children()[cont].get_children()[0].pack_start(tmp)
+                        else:
+                            pass
+                    # si metemos la informacion de que procesador es, metemos imagen correspondiente
+                    elif i == 4:
+                        tmp = gtk.Image()
+                        if self.info[i] == "centrino":
+                            tooltip.set_tip(self.box.get_children()[cont], "Centrino detectado")
+                            tmp.set_from_file(os.path.join(pglobals.image_dir,'centrino_46.png'))
+                            self.box.get_children()[cont].get_children()[0].pack_start(tmp)
+                    else:
+                        fd = self.define_font()
+                        tmp = gtk.Label(self.info[i])
+                        tmp.modify_font(fd)
+                        self.box.get_children()[cont].get_children()[0].pack_start(tmp)
+        
         self.applet.show_all()
         return 1
 
@@ -157,6 +165,9 @@ class pycentrinoApplet(gnome.applet.Applet):
         self.big_evbox.add(self.box)
         self.applet.show_all()
 
+    def on_okbutton2_clicked(self,widget):
+        self.preferences.hide()
+
      # this callback is to create the context menu
     def button_clicked(self,widget,event):
         if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
@@ -172,7 +183,7 @@ class pycentrinoApplet(gnome.applet.Applet):
     # and this one to show the about box   
     def about_info(self,event,data=None):
         about = gnome.ui.About("CPU performance stats applet",pglobals.version,"GPL",\
-                               "GNOME Applet to check throttling CPU states",["Arturo Gonzalez <arturogf@ugr.es>"],\
+                               "GNOME Applet to check throttling CPU states",["Arturo Gonzalez <arturogf@ugr.es>,Lorenzo Gil <lgs@sicem.biz>"],\
                                ["Arturo Gonzalez <arturogf@ugr.es>"],"Arturo Gonzalez <arturogf@ugr.es>",self.logo_pixbuf)
         about.show()
 
@@ -264,8 +275,6 @@ class pycentrinoApplet(gnome.applet.Applet):
                 self.info[3] = line.strip()
 
             f_sys.close()
-   
-
         else:
             self.info[3] = ""
 
@@ -278,8 +287,6 @@ class pycentrinoApplet(gnome.applet.Applet):
         else:
             self.info[4] = ""
 
-        
-
     # this callback is to create a context menu with the second button
     def create_menu(self):
         self.applet.setup_menu(self.propxml,self.verbs,None)
@@ -307,7 +314,7 @@ class pycentrinoApplet(gnome.applet.Applet):
     def on_checkbutton5_toggled(self,widget):
         self.checks[4] = widget.get_active()
 
-    def on_checkbutton5_toggled(self,widget):
+    def on_checkbutton6_toggled(self,widget):
         self.checks[5] = widget.get_active()
 
 
@@ -319,6 +326,10 @@ class pycentrinoApplet(gnome.applet.Applet):
 
         # instance variable section
         self.info = ["T0","0%","C1","powersave","centrino",""]
+        # history is a queue and will be managed with append() and pop()
+        self.history_max = 20
+        self.history = []
+        self.history.append(list(self.info))
         self.timeout_interval = 1000        
         self.throttling_states = 0
         self.tooltip_text = ""
@@ -345,7 +356,6 @@ class pycentrinoApplet(gnome.applet.Applet):
 
         # create the applet and the widgets
         self.applet = applet
-        #self.tooltips = gtk.Tooltips()
         self.big_evbox = gtk.EventBox()
         self.big_evbox.connect("button-press-event",self.button_press)
         self.orientation = self.applet.get_orient()
@@ -364,44 +374,61 @@ class pycentrinoApplet(gnome.applet.Applet):
         cont = -1
         # create the applet with little hboxes containing each info checked in the preferences
         for i in range(len(self.checks)):
+            
             if self.checks[i]:
                 cont = cont + 1
                 self.box.pack_start(gtk.EventBox())
-
+                
                 # add un hbox para meter contenido
                 self.box.get_children()[cont].add(gtk.HBox())
                 # metemos una etiqueta con un espacio
-                tmp = gtk.Label(" ")
-                self.box.get_children()[cont].get_children()[0].pack_start(tmp)
+                self.box.get_children()[cont].get_children()[0].pack_start(gtk.Label(" "))
 
                 cont = cont +1
                 self.box.pack_start(gtk.EventBox())
+                tooltip = gtk.Tooltips()
                 
                 # add un hbox para meter contenido
                 self.box.get_children()[cont].add(gtk.HBox())
                 if i == 3:
-                    if self.info[i] == "powersave":
-                        tmp = gtk.Image()
+                    tmp = gtk.Image()
+                    if self.info[i] == "":
+                        tooltip.set_tip(self.box.get_children()[cont], "no tiene soporte para scaling_governor")
+                        tmp.set_from_file(os.path.join(pglobals.image_dir,'warning.png'))
+                        self.box.get_children()[cont].get_children()[0].pack_start(tmp)
+                    elif self.info[i] == "powersave":
+                        tooltip.set_tip(self.box.get_children()[cont], "ahorro de energia")
                         tmp.set_from_file(os.path.join(pglobals.image_dir,'powersave.png'))
                         self.box.get_children()[cont].get_children()[0].pack_start(tmp)
                     elif self.info[i] == "performance":
-                        tmp = gtk.Image()
+                        tooltip.set_tip(self.box.get_children()[cont], "energia al maximo")
                         tmp.set_from_file(os.path.join(pglobals.image_dir,'performance.png'))
                         self.box.get_children()[cont].get_children()[0].pack_start(tmp)
                     elif self.info[i] == "userspace":
-                        tmp = gtk.Image()
+                        tooltip.set_tip(self.box.get_children()[cont], "Usando modo usuario")
                         tmp.set_from_file(os.path.join(pglobals.image_dir,'userspace.png'))
                         self.box.get_children()[cont].get_children()[0].pack_start(tmp)
-
-                # si metemos la informacin de que procesador es, metemos imagen correspondiente
+                    else:
+                        pass
+                # si metemos la informacion de que procesador es, metemos imagen correspondiente
                 elif i == 4:
+                    tmp = gtk.Image()
                     if self.info[i] == "centrino":
-                        tmp = gtk.Image()
+                        tooltip.set_tip(self.box.get_children()[cont], "Centrino detectado")
                         tmp.set_from_file(os.path.join(pglobals.image_dir,'centrino_46.png'))
                         self.box.get_children()[cont].get_children()[0].pack_start(tmp)
+                    elif self.info[i] == "speedstep":
+                        tooltip.set_tip(self.box.get_children()[cont], "Speedstep detectado")
+                        tmp.set_from_file(os.path.join(pglobals.image_dir,'speedstep_46.png'))
+                        self.box.get_children()[cont].get_children()[0].pack_start(tmp)
+                    else:
+                        tooltip.set_tip(self.box.get_children()[cont], "Centrino/Speedstep not found")
+                        tmp.set_from_file(os.path.join(pglobals.image_dir,'warning.png'))
+                        self.box.get_children()[cont].get_children()[0].pack_start(tmp)
                 else:
                     fd = self.define_font()
                     tmp = gtk.Label(self.info[i])
+                    tmp.modify_font(fd)
                     self.box.get_children()[cont].get_children()[0].pack_start(tmp)
 
         self.box.show()
